from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import math


#
# Sensor for a clock.
#
# Blatantly modified by Tuomas Kuosmanen <tigert@ximian.com> to use a nifty
# clock face I "found from the internet" :^) - basically a test on how to 
# customize these, and it looked nice so I thought I'd share.
#
# This represents the RAF wartime briefing room clock for pilots as far as
# I know, a nice colorful clock face ;-)
#

class ClockSensor(Sensor):

    HAND_HOUR = [-0.03, -0.2,
                 0.03, -0.2,
                 0.03, 0.6,
                 -0.03, 0.6]

    HAND_MINUTE = [-0.03, -0.2,
                   0.03, -0.2,
                   0.03, 0.9,
                   -0.03, 0.9]

    HAND_SECOND = [-0.02, -0.2,
                   0.02, -0.2,
                   0.02, 1.0,
                   -0.02, 1.0]


    def __init__(self):

        global _; _ = i18n.Translator("clock-sensor")

        Sensor.__init__(self)
        self._set_config_type("show-seconds", TYPE_BOOL, 1)
        
        self._add_timer(0, self.__get_data)



    #
    # Returns the configuration GUI.
    #
    # TODO: add configuration option for UTC time
    #
    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name(_("Clock"))
        configurator.add_title(_("Clock Settings"))
        configurator.add_checkbox(_("_Show seconds"), "show-seconds",
                              _("Toggles the seconds on and off"))
        
        return configurator



    #
    # Rotates the given point by the given angle.
    #
    def __rotate(self, x, y, angle):

        cos_a = math.cos(angle)
        sin_a = math.sin(angle)
        rx = x * cos_a - y * sin_a
        ry = y * cos_a + x * sin_a

        return (rx, ry)



    #
    # Generates and returns the drawing instructions for a clock hand.
    #
    def __do_hand(self, points, angle):

        p = ""
        for i in xrange(0, len(points), 2):
            x = points[i]
            y = points[i + 1]
            rx, ry = self.__rotate(x, y, angle)
            p += " %(rx)f %(ry)f" % vars()
        #end for

        return p



    def __get_data(self):

        current_time = time.localtime(time.time())

        hour = current_time[3]
        minute = current_time[4]
        second = current_time[5]

        h_angle = -(2 * math.pi) / 12 * (hour + (minute / 60.0))
        m_angle = -(2 * math.pi) / 60 * minute
        s_angle = -(2 * math.pi) / 60 * second

        gfx_hour = self.__do_hand(self.HAND_HOUR, h_angle)
        gfx_minute = self.__do_hand(self.HAND_MINUTE, m_angle)
        if (self._get_config("show-seconds")):
            gfx_second = self.__do_hand(self.HAND_SECOND, s_angle)
        else:
            gfx_second = ""

        signal = self._new_output()
        signal.set("date", self.__get_date(current_time))
        signal.set("hours", "%(hour)02d" % vars())
        signal.set("minutes", "%(minute)02d" % vars())
        signal.set("seconds", "%(second)02d" % vars())
        signal.set("hands", "clear, color black, "
                            "polygon %(gfx_hour)s 1, "
                            "polygon %(gfx_minute)s 1,"
                            "color red,"
                            "polygon %(gfx_second)s 1" % vars())
        self._send_output(signal)

        self._add_timer(1000, self.__get_data)



    def __get_date(self, current_time):

        day = current_time[2]
        month = current_time[1]
        year = current_time[0]
        date = "%(day)d.%(month)d.%(year)d" % vars()

        return date


def new_sensor(args): return apply(ClockSensor, args)
